;
; Swedish Payfone "Smart-Card Emulator" v8.0A1
; (p) 1993 Beeblebrox Industries Unlimited
;
;       Port Useage List:
;       =================
;       PA0 - EEPROM Data Out
;       PA1 - EEPROM Data In
;       PA2 - EEPROM Serial Clock
;       PA3 - EEPROM Chip Select
;       PA4 - Write Enable (Input, Hi-Active)
;       PA5 - Reset (Input, Hi-Active)
;       PA6 - Data (Output)
;       PA7 - VPP (Input, Hi-Active)
;       PB1 - Card Unit-Counter Reset (Input, Hi-Active)
;       INT - Clock (Inverted).

PORTA:          EQU     $00
PORTB:          EQU     $01
DDRA:           EQU     $04
DDRB:           EQU     $05
PDRA:           EQU     $10
PDRB:           EQU     $11
TCR:            EQU     $08
TDR:            EQU     $09
IRQSR:          EQU     $0A
RAM:            EQU     $E0
EPROM:          EQU     $200
MOR:            EQU     $17
COP:            EQU     $3F0
TIRVEC:         EQU     $3F8
EXTVEC:         EQU     $3FA
SWIVEC:         EQU     $3FC
RESVEC:         EQU     $3FE


	ORG     RAM             ; Ram Variables

BITCNT:         DS      1       ; Card Read Bitcounter
FUSCNT:         DS      1       ; Cards Fused-Unit Counter
BYTE:           DS      1       ; Temporary Byte Storage
EEADR:          DS      1       ; EEPROM Address Counter
EEBYTE:         DS      1       ; EEPROM Command Storage
EEDATA1:        DS      1       ; EEPROM MSB
EEDATA2:        DS      1       ; EEPROM LSB
CARTE:          DS      12      ; Card Data RAM Storage Locations

	ORG     TIRVEC
	DW      START           ; Timer Interrupt
	ORG     EXTVEC
	DW      INT             ; External Interrupt
	ORG     SWIVEC
	DW      START           ; Software Interrupt
	ORG     RESVEC
	DW      START           ; Reset Vector....

	ORG     EPROM           ; Here we go again...........

START:  SEI                     ; Disable Interrupts    
	LDA     #%01001110      ; Define PORT A Data Directions
	STA     DDRA
	LDA     #%11111101      ; Define PORT B Data Directions
	STA     DDRB
	CLR     BITCNT          ; Clear bitcounter.
	CLR     EEADR           ; Clear EEPROM Address Counter

	CLRX                    ; Reset Byte Counter
CCOPY:  STX     BYTE            ; Store byte-counter...
	JSR     EEREAD          ; Read 16-bit word from EEPROM
	LDX     BYTE            ; Restore byte-counter
	LDA     EEDATA1         ; Get EEDATA MSB
	STA     CARTE,X         ; Store away card data!
	INCX                    ; Next byte!!!
	LDA     EEDATA2         ; Get EEDATA LSB
	STA     CARTE,X         ; Store away card data!
	INCX                    ; Next byte!!!
	INC     EEADR           ; Increase EEPROM Address Counter
	CPX     #$0C            ; Have we copied 12 bytes of card data yet?
	BLO     CCOPY           ; If not, looping time...

	JSR     EE_WE           ; EEPROM Write Enable...
	LDA     #$FF            ; Set EEPROM Address counter to $FF
	STA     EEADR           ; And store it at EEADR
	LDA     #$02            ; Reset number of fused units on card...
	STA     EEDATA1         ; ...and store this on EEDATA 1/2
	CLR     EEDATA2
	BRCLR   1,PORTB,NOFRST  ; Check if reset button is pressed down!
	JSR     EEWRT           ; And if it is, write $02 to EEPROM adr. $FF
	BRA     FUSRST          ; Branch to main...

NOFRST: JSR     EEREAD          ; Else, read fused units address from EEPROM
FUSRST: LDA     EEDATA1         ; Store fused units count in RAM.
	STA     FUSCNT
	
	CLI                     ; Enable Interrupts

MAIN:   BRA     MAIN            ; Here we go... Again, again and again!!!

INT:    BSET    6,PORTA         ; First bit always "1"
	BRSET   5,PORTA,NORESET ; Check for RESET Low condition
	LDA     BITCNT
	CLR     BITCNT          ; Reset Counter
	INC     BITCNT          ; Increase Counter(?)
	RTI

NORESET:LDA     BITCNT          ; Store Bitpointer in TEMP1
	CMP     #$60            ; Is it credits or serial #?
	BHS     CREDIT          ; If credits, skip this part!
	LSRA                    ; Right is the way to go....
	LSRA                    ; But two rights can make one wrong maybe?
	LSRA                    ; So that is why we use three of them!
	TAX                     ; Acc. transfer to Index Reg!
	LDA     CARTE,X         ; Get byte!
	STA     BYTE            ; Store byte too...
	LDA     BITCNT          ; Get bitpointer
	AND     #%00000111      ; Mask off bits 7-3
	TAX                     ; Get number of rotations...
	LDA     BYTE            ; Get current databyte...
ROTL:   CPX     #00             ; Check if it's zero?
	BEQ     ENDROT          ; If it is, quit shiftin left!
	LSLA                    ; Shift Left one step
	DECX                    ; Decrement Index reg
	BRA     ROTL            ; ...And here we go again!
ENDROT: AND     #%10000000      ; Mask of the seven low bits...
	CMP     #%10000000      ; Is the current bit set?
	BEQ     BITSET
	BCLR    6,PORTA
BITSET: INC     BITCNT          ; Move one bit further on....
	RTI

CREDIT: BRCLR   4,PORTA,NOWRT   ; Is it write time?
	BRCLR   5,PORTA,NOWRT   ; Check for RESET low!
;       BRCLR   7,PORTA,NOWRT   ; Check for VPP = 21V
	INC     FUSCNT          ; Increase used up credits
	INC     BITCNT          ; Increase bitcounter too!
	LDA     FUSCNT          ; Get number of fused units...
	STA     EEDATA1         ; Store in EEDATA MSB
	CLR     EEDATA2         ; Clear EEDATA LSB
	LDA     #$FF            ; EEADR --> $FF (Units Counter)
	STA     EEADR           ; Set EEADR
	JSR     EEWRT           ; Execute EEPROM Write Command
	RTI                     ; Return from interupt!

NOWRT:  SUB     #$60            ; Subtract $60 to get to the units area!
	CMP     FUSCNT          ; Compare with number of fused units
	BLO     NOTWRT          ; If it is lower, leave output as "1".
	BCLR    6,PORTA         ; Else, set the output to "0".
NOTWRT: INC     BITCNT          ; Increase bitcounter
	RTI                     ; return from interupt!

EEREAD: BSET    3,PORTA         ; EEPROM Chip Select
	LDA     #%11000000      ; EEPROM Read Address CMD!
	LDX     #$3             ; Command is 3 bits long
	JSR     EECMD           ; Call CMD Output...
	LDA     EEADR           ; Get EEPROM Address
	LDX     #$08            ; And it's 8 bits...
	JSR     EECMD           ; Execute Command!
	CLRA                    ; Clear Acc.
	LDX     #$08            ; Read first 8 bits from EEPROM
RLOOP:  LSLA                    ; Shift acc. left one step...
	BSET    2,PORTA         ; EEPROM Clock High...
	BRCLR   0,PORTA,NOTE1   ; Check for EEDATA Status
	ORA     #$01            ; Set bit 0 to "1" status...
NOTE1:  BCLR    2,PORTA         ; EEPROM Clock Low...
	DECX                    ; Decrement bits left to read
	CPX     #00             ; Have we read all bits yet?
	BHI     RLOOP           ; If not, keep goin'
	STA     EEDATA1         ; Store EEDATA MSB
	CLRA                    ; Clear Acc.
	LDX     #$08            ; Read second 8 bit block from EEPROM
RLOOP2: LSLA                    ; Shift acc. left one step...
	BSET    2,PORTA         ; EEPROM Clock High...
	BRCLR   0,PORTA,NOTE2   ; Check EEDATA Status
	ORA     #$01            ; Set bit 0 to "1" status...
NOTE2:  BCLR    2,PORTA         ; EEPROM Clock Low...
	DECX                    ; Decrement bits left to read
	CPX     #00             ; Have we read all bits yet?
	BHI     RLOOP2          ; If not, keep goin'
	STA     EEDATA2         ; Store EEDATA LSB
	BCLR    3,PORTA         ; EEPROM Chip Select Low
	RTS                     ; Mission Accomplished!

EEWRT:  BSET    3,PORTA         ; EEPROM Chip Select
	LDA     #%10100000      ; EEPROM Write Command
	LDX     #$03            ; Command is 3 bits long
	JSR     EECMD           ; Execute Command!
	LDA     EEADR           ; Get EEPROM Address
	LDX     #$08            ; 8 Bits long...
	JSR     EECMD           ; Execute Command!
	LDA     EEDATA1         ; Get EEDATA MSB
	LDX     #$08            ; 8 Bits to send
	JSR     EECMD           ; Execute Command
	LDA     EEDATA2         ; Get EEDATA LSB
	LDX     #$08            ; 8 Bits to send
	JSR     EECMD           ; Execute Command!
	BCLR    3,PORTA         ; EEPROM Chip Select Low
	RTS                     ; Mission Accomplished!

EE_WE:  BSET    3,PORTA         ; EEPROM Chip Select
	LDA     #%10000000      ; EEPROM Write Enable/Disable Command
	LDX     #$03            ; And it's 3 bits long.
	JSR     EECMD           ; Execute Command!
	LDA     #%11000000      ; EEPROM Write Enable...
	LDX     #$08            ; 8 Bits to send
	JSR     EECMD           ; Execute Command!
	BCLR    3,PORTA         ; EEPROM Chip Select Low
	RTS                     ; Mission Accomplished!

EE_WD:  BSET    3,PORTA         ; EEPROM Chip Select
	LDA     #%10000000      ; EEPROM Write Enable/Disable Command
	LDX     #$03            ; 3 bits long...
	JSR     EECMD           ; Execute Command!
	LDA     #%00000000      ; EEPROM Write Disable
	LDX     #$08            ; 8 bits to send!
	JSR     EECMD           ; Execute Command!
	BCLR    3,PORTA         ; EEPROM Chip Select Low
	RTS                     ; Mission Accomplished!

EECMD:  STA     EEBYTE          ; Store Data to be Processed...
EELOOP: BSET    1,PORTA         ; Set DI High on EEPROM
	BRSET   7,EEBYTE,EEONE  ; Check for EEBYTE bit 7 High...    
	BCLR    1,PORTA         ; If not, set DI to low!
EEONE:  BSET    2,PORTA         ; EEPROM Clock High!        
	NOP                     ; Delay 500 ns
	BCLR    2,PORTA         ; EEPROM Clock Low!
	LSL     EEBYTE          ; Shift Data Left...
	DECX                    ; Decrement bitcounter
	CPX     #00             ; Are we done yet?
	BHI     EELOOP          ; No? Then get on with the work...
	RTS                     ; Mission Accomplished!

	END

