;
; Swedish Payfone "Smart-Card Emulator" v8.0A2
; (p) 1993 Beeblebrox Industries Unlimited
;
;       Port Useage List:
;       =================
;       PA0 - EEPROM Data Out
;       PA1 - EEPROM Data In
;       PA2 - EEPROM Serial Clock
;       PA3 - EEPROM Chip Select
;       PA4 - Write Enable (Input, Hi-Active)
;       PA5 - Reset (Input, Hi-Active)
;       PA6 - Data (Output)
;       PA7 - VPP (Input, Hi-Active)
;       PB1 - Card Unit-Counter Reset (Input, Hi-Active)
;       INT - Clock (Inverted).

PORTA:          EQU     $00
PORTB:          EQU     $01
DDRA:           EQU     $04
DDRB:           EQU     $05
PDRA:           EQU     $10
PDRB:           EQU     $11
TCR:            EQU     $08
TDR:            EQU     $09
IRQSR:          EQU     $0A
RAM:            EQU     $E0
EPROM:          EQU     $200
MOR:            EQU     $17
COP:            EQU     $3F0
TIRVEC:         EQU     $3F8
EXTVEC:         EQU     $3FA
SWIVEC:         EQU     $3FC
RESVEC:         EQU     $3FE


	ORG     RAM             ; Ram Variables

BITCNT:         DS      1
FUSCNT:         DS      1
BYTE:           DS      1
CARDNO:         DS      1
EEADR:          DS      1
EEDATA1:        DS      1
EEDATA2:        DS      1
CARTE:          DS      12

	ORG     TIRVEC
	DW      START           ; Timer Interrupt
	ORG     EXTVEC
	DW      INT             ; External Interrupt
	ORG     SWIVEC
	DW      START           ; Software Interrupt
	ORG     RESVEC
	DW      START           ; Reset Vector....

	ORG     EPROM           ; Here we go again...........

START:  SEI                     ; Disable Interrupts    
	LDA     #%01001110
	STA     DDRA
	LDA     #%11111100
	STA     DDRB

	LDA     #$FF        
	STA     EEADR
	JSR     EEREAD
	BRCLR   1,PORTB,NOFRST  ; Check for card unit reset...
	LDA     #$02
	STA     EEDATA1
	INC     EEDATA2
	LDA     EEDATA2
	CMP     #$29
	BLS     NOFRST
	CLR     EEDATA2
NOFRST: LDA     EEDATA1
	STA     FUSCNT
	LDA     EEDATA2
	CMP     #$29            ; Check for Card # in range 0-41
	BLS     CNUMOK2
	CLRA
CNUMOK2: 
	STA     CARDNO
	LDA     EEDATA2
	LDX     #$06
	MUL
	STA     EEADR

	CLRX
CCOPY:  STX     BITCNT
	JSR     EEREAD
	LDX     BITCNT
	LDA     EEDATA1
	STA     CARTE,X
	INCX
	LDA     EEDATA2
	STA     CARTE,X
	INCX
	INC     EEADR
	CPX     #$0C
	BLO     CCOPY
	CLR     BITCNT
	JSR     EE_WE

	BRCLR   1,PORTB,NOFRST2
	LDA     #$FF
	STA     EEADR
	LDA     FUSCNT
	STA     EEDATA1
	LDA     CARDNO
	STA     EEDATA2
	JSR     EEWRT

NOFRST2:
	CLI                     ; Enable Interrupts

MAIN:   BRA     MAIN

INT:    BSET    6,PORTA         ; First bit always "1"
	BRSET   5,PORTA,NORESET ; Check for RESET Low condition
	LDA     BITCNT
	CLR     BITCNT          ; Reset Counter
	INC     BITCNT
	RTI

NORESET:LDA     BITCNT          ; Store Bitpointer in TEMP1
	CMP     #$60            ; Is it credits or serial #?
	BHS     CREDIT          ; If credits, skip this part!
	LSRA                    ; Right is the way to go....
	LSRA                    ; But two rights can make one wrong maybe?
	LSRA                    ; So that is why we use three of them!
	TAX                     ; Acc. transfer to Index Reg!
	LDA     CARTE,X         ; Get byte!
	STA     BYTE            ; Store byte too...
	LDA     BITCNT          ; Get bitpointer
	AND     #%00000111      ; Mask off bits 7-3
	TAX                     ; Get number of rotations...
	LDA     BYTE            ; Get current databyte...
ROTL:   CPX     #00             ; Check if it's zero?
	BEQ     ENDROT          ; If it is, quit shiftin left!
	LSLA                    ; Shift Left one step
	DECX                    ; Decrement Index reg
	BRA     ROTL            ; ...And here we go again!
ENDROT: AND     #%10000000      ; Mask of the seven low bits...
	CMP     #%10000000      ; Is the current bit set?
	BEQ     BITSET
	BCLR    6,PORTA
BITSET: INC     BITCNT          ; Move one bit further on....
	RTI

CREDIT: BRCLR   4,PORTA,NOWRT   ; Is it write time?
	BRCLR   5,PORTA,NOWRT   ; Check for RESET low!
;       BRCLR   7,PORTA,NOWRT   ; Check for VPP = 21V
	INC     FUSCNT          ; Increase used up credits
	INC     BITCNT          ; Increase bitcounter too!
	LDA     FUSCNT
	STA     EEDATA1
	LDA     CARDNO
	STA     EEDATA2
	LDA     #$FF
	STA     EEADR
	JSR     EEWRT
	RTI                     ; Return from interupt

NOWRT:  SUB     #$60
	CMP     FUSCNT
	BLO     NOTWRT
	BCLR    6,PORTA
NOTWRT: INC     BITCNT
	RTI

EEREAD: BSET    3,PORTA
	LDA     #%11000000      ; EEPROM Read Address CMD!
	LDX     #$3
	JSR     EECMD           ; Call CMD Output...
	LDA     EEADR           ; Get EEPROM Address
	LDX     #$08            ; And it's 8 bits...
	JSR     EECMD
	CLRA
	LDX     #$08
RLOOP:  LSLA
	BSET    2,PORTA
	BRCLR   0,PORTA,NOTE1
	ORA     #$01
NOTE1:  BCLR    2,PORTA
	DECX
	CPX     #00
	BHI     RLOOP
	STA     EEDATA1
	CLRA
	LDX     #$08
RLOOP2: LSLA
	BSET    2,PORTA
	BRCLR   0,PORTA,NOTE2
	ORA     #$01
NOTE2:  BCLR    2,PORTA
	DECX
	CPX     #00
	BHI     RLOOP2
	STA     EEDATA2
	BCLR    3,PORTA
	RTS

EEWRT:  BSET    3,PORTA
	LDA     #%10100000
	LDX     #$03
	JSR     EECMD
	LDA     EEADR
	LDX     #$08
	JSR     EECMD
	LDA     EEDATA1     
	LDX     #$08
	JSR     EECMD
	LDA     EEDATA2
	LDX     #$08
	JSR     EECMD
	BCLR    3,PORTA
	RTS

EE_WE:  BSET    3,PORTA
	LDA     #%10000000
	LDX     #$03
	JSR     EECMD
	LDA     #%11000000
	LDX     #$08
	JSR     EECMD
	BCLR    3,PORTA
	RTS

EE_WD:  BSET    3,PORTA
	LDA     #%10000000
	LDX     #$03
	JSR     EECMD
	LDA     #%00000000
	LDX     #$08
	JSR     EECMD
	BCLR    3,PORTA
	RTS

EECMD:  STA     BYTE
EELOOP: BSET    1,PORTA         ; Set DI High on EEPROM
	BRSET   7,BYTE,EEONE    ; Check for EEBYTE bit 7 High...    
	BCLR    1,PORTA         ; If not, set DI to low!
EEONE:  BSET    2,PORTA        
	NOP
	BCLR    2,PORTA
	LSL     BYTE
	DECX
	CPX     #00
	BHI     EELOOP
	RTS

	END

